module Render (..) where

import Text
import String exposing (..)
import Color exposing (..)
import Graphics.Collage exposing (..)
import Graphics.Element exposing (..)
import Data exposing (..)
import Powerup exposing (..)


render : GameState -> ( Int, Int ) -> Element
render { gameOver, state, player, ballsList, bricksList, levelNum, fallingPowerups, activePowerups } ( width, height ) =
  Graphics.Collage.collage
    (gameWidth + borderWidth)
    (gameHeight + borderWidth)
    [ rect (gameWidth + borderWidth) (gameHeight + borderWidth)
        |> filled black
    , drawBorders borders player.warp
    , drawPowerupList fallingPowerups
    , drawObject player
    , drawActivePowerups activePowerups
    , drawBricksList bricksList
    , if not gameOver then
        drawBallsList ballsList
      else
        toForm empty
    , if gameOver then
        group
          [ gameOverMsg
          , append (toString (round player.score)) " Points"
              |> Text.fromString
              |> Text.height 40
              |> Text.monospace
              |> leftAligned
              |> toForm
              |> move ( 0, -45 )
          , if player.lives > 0 then
              "You win!"
                |> Text.fromString
                |> Text.height 40
                |> Text.monospace
                |> leftAligned
                |> toForm
                |> move ( 0, -100 )
            else
              toForm empty
          ]
      else
        toForm empty
    , writeText (append "lives " (toString player.lives))
        |> move ( halfWidth - borderWidth, halfHeight + borderWidth / 4 )
    , Text.append
        (append "" (toString (round player.score))
          |> Text.fromString
          |> (Text.color black)
        )
        (append "score " (toString (round player.score))
          |> Text.fromString
          |> Text.color white
        )
        |> Text.monospace
        |> Text.height (borderWidth / 2 - 1)
        |> leftAligned
        |> toForm
        |> move ( borderWidth * 1 / 2 - 10 - halfWidth + borderWidth / 2, halfHeight + borderWidth / 4 )
    , writeText (append "level " (toString levelNum))
        |> move ( 0, halfHeight + borderWidth / 4 )
    ]
    |> container width height middle


drawActivePowerups : List Powerup -> Form
drawActivePowerups powerups =
  drawActivePowerups' powerups 0
    |> drawPowerupList


drawActivePowerups' : List Powerup -> Float -> List Powerup
drawActivePowerups' powerups which =
  case powerups of
    powerup :: restPowerups ->
      { powerup
        | coord = ( -halfWidth - borderWidth / 4, which * borderWidth / 2 - borderWidth / 4 - halfHeight )
        , shape = Rectangle ( borderWidth / 2 - 5, borderWidth / 2 - 5 )
      }
        :: (drawActivePowerups' restPowerups (which + 1))

    [] ->
      []


drawPowerupList : List Powerup -> Form
drawPowerupList powerups =
  List.map drawPowerup powerups
    |> group


drawPowerup powerup =
  let
    size =
      case powerup.shape of
        Circle rad ->
          rad

        Rectangle ( x, y ) ->
          x
  in
    group
      [ drawElement powerup.shape
          |> filled powerup.colour
      , drawElement powerup.shape
          |> outlined (solid backgroundColour)
      , powerup.name
          |> Text.fromString
          |> Text.color powerup.textColour
          |> Text.height (size * 0.8)
          |> leftAligned
          |> toForm
      ]
      |> move powerup.coord


invertColour : Color -> Color
invertColour colour =
  let
    temp =
      toRgb colour
  in
    rgba (255 - temp.red) (255 - temp.green) (255 - temp.blue) (temp.alpha)


gameOverMsg =
  "GAME OVER"
    |> Text.fromString
    |> Text.height 80
    |> \txt ->
        group [ text txt, outlinedText (solid white) txt ]
          |> move ( 0, 30 )


writeText : String -> Form
writeText string =
  string
    |> Text.fromString
    |> Text.monospace
    |> Text.color white
    |> Text.height (borderWidth / 2 - 1)
    |> leftAligned
    |> toForm


drawBorders : Borders -> Bool -> Form
drawBorders { width, height, colour } warp =
  group
    [ Graphics.Collage.rect (width) (height + borderWidth)
        |> filled backgroundColour
        |> move ( 0, -borderWidth / 2 )
    , if warp then
        Graphics.Collage.rect (40) (halfHeight * 3 / 2)
          |> gradient (linear ( 0, 0 ) ( 20, 0 ) [ ( 1, lightOrange ), ( 0, setAlpha 0 lightOrange ) ])
          |> move ( halfWidth - 20, halfHeight / 4 )
          |> rotate (degrees 0)
      else
        toForm empty
    , if warp then
        Graphics.Collage.rect (40) (halfHeight * 3 / 2)
          |> gradient (linear ( 0, 0 ) ( 20, 0 ) [ ( 1, lightBlue ), ( 0, setAlpha 0 lightBlue ) ])
          |> move ( -halfWidth + 20, halfHeight / 4 )
          |> rotate (degrees 180)
      else
        toForm empty
    ]


setAlpha : Float -> Color -> Color
setAlpha num col =
  let
    rgb =
      toRgb col
  in
    rgba rgb.red rgb.green rgb.blue num


drawBricksList : List Brick -> Form
drawBricksList bricks =
  List.map drawBrick bricks
    |> group


drawBallsList : List Ball -> Form
drawBallsList balls =
  List.map drawBall balls
    |> group


drawBrick : Brick -> Form
drawBrick brick =
  let
    shape =
      drawElement brick.shape
  in
    group
      [ shape |> filled brick.colour |> setOpiacity brick.armor brick.maxArmor
      , -- shape |> outlined (solid black) ,
        shape |> drawBrickArmor brick.armor brick.maxArmor
      ]
      |> move brick.coord


setOpiacity : Int -> Int -> Form -> Form
setOpiacity armor maxArmor shape =
  alpha (1 - (Basics.toFloat (maxArmor - armor)) / (Basics.toFloat maxArmor)) shape


drawBrickArmor : Int -> Int -> Shape -> Form
drawBrickArmor armor maxArmor shape =
  if armor == maxArmor then
    outlined (solid blue) shape
  else
    let
      lineStyle =
        case armor of
          3 ->
            solid blue

          2 ->
            dashed blue

          1 ->
            dotted blue |> \ls -> { ls | dashOffset = 20 }

          otherwise ->
            solid blue
    in
      outlined (lineStyle) shape


drawObject : Object a -> Form
drawObject object =
  drawElement object.shape
    |> filled object.colour
    |> move object.coord


drawBall : Ball -> Form
drawBall ball =
  group
    [ drawElement ball.shape
        |> filled ball.colour
    , if ball.godmode then
        drawHalo
      else
        toForm empty
    ]
    |> move ball.coord


drawHalo : Form
drawHalo =
  (List.map
    (\x ->
      Graphics.Collage.circle x
        |> \y -> outlined (solid (getGradient (round x))) y
    )
    (List.map (\x -> x * 2) [1..25])
  )
    --[3,6,9,12,15,18,21,24,27,30,33,36,39,42,45] )
    |>
      group


getGradient : Int -> Color
getGradient num =
  rgba (255) (num * 5) (0) (Basics.toFloat num |> \x -> 1 - x / 50)


drawElement : ShapeType -> Shape
drawElement shape =
  case shape of
    Circle radius ->
      Graphics.Collage.circle radius

    Rectangle coord ->
      uncurry Graphics.Collage.rect coord
